@@ -1,5 +1,9 @@
 //! Compile-time assertions to ensure that invariants are met.
 //!
+//! _All_ assertions within this crate are performed at **compile-time**. This
+//! allows for finding errors quickly and early when it comes to ensuring
+//! certain features or aspects of a codebase.
+//!
 //! # Usage
 //!
 //! This crate is available [on crates.io][crate] and can be used by adding the
@@ -7,7 +11,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! static_assertions = "0.2.4"
+//! static_assertions = "0.2.5"
 //! ```
 //!
 //! and this to your crate root:
@@ -18,12 +22,32 @@
 //! # fn main() {}
 //! ```
 //!
+//! # Examples
+//!
+//! Very thorough examples are provided in the docs for each individual macro.
+//! Failure case examples are also documented.
+//!
 //! # Limitations
 //!
 //! Due to implementation details, some macros can only be used normally from
 //! within the context of a function. To use these macros in other contexts, a
 //! unique label must be provided.
 //!
+//! ```compile_fail
+//! # #[macro_use] extern crate static_assertions;
+//! # fn main() {}
+//! // error: expected item after attributes
+//! const_assert!(true == true);
+//! ```
+//!
+//! This can be fixed via:
+//!
+//! ```
+//! # #[macro_use] extern crate static_assertions;
+//! # fn main() {}
+//! const_assert!(label; true == true);
+//! ```
+//!
 //! This issue can be followed [here][issue1]. Feedback and potential solutions
 //! are welcome!
 //!
@@ -37,7 +61,7 @@
 #[doc(hidden)]
 pub extern crate core as _core;
 
-/// Asserts that the configuration is set.
+/// Asserts that a given configuration is set.
 ///
 /// # Examples
 ///
@@ -64,6 +88,15 @@ pub extern crate core as _core;
 ///                 any(    feature = "mysql", feature = "mongodb")));
 /// # fn main() {}
 /// ```
+///
+/// We can't be compiling for both Unix _and_ Windows simultaneously:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// assert_cfg!("No, that's not how it works! ಠ_ಠ", all(unix, windows));
+/// # }
+/// ```
 #[macro_export]
 macro_rules! assert_cfg {
     () => {};
@@ -77,7 +110,7 @@ macro_rules! assert_cfg {
     };
 }
 
-/// Asserts at compile-time that the types have equal sizes.
+/// Asserts that types are equal in size.
 ///
 /// When performing operations such as pointer casts or dealing with [`usize`]
 /// versus [`u64`] versus [`u32`], the size of your types matter. This is where
@@ -89,7 +122,7 @@ macro_rules! assert_cfg {
 /// [`assert_eq_size_ptr`](macro.assert_eq_size_ptr.html). Instead of specifying
 /// types to compare, values' sizes can be directly compared against each other.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```
 /// # #[macro_use]
@@ -97,18 +130,22 @@ macro_rules! assert_cfg {
 /// // Can be declared outside of a function if labeled
 /// assert_eq_size!(bytes; (u8, u8), u16);
 ///
-/// // Fails to compile (same label):
-/// // assert_eq_size!(bytes; u8, u8);
-///
 /// fn main() {
 ///     // Supports unlimited arguments:
 ///     assert_eq_size!([u8; 4], (u16, u16), u32);
-///
-///     // Produces a compilation failure:
-///     // assert_eq_size!(u32, u8);
 /// }
 /// ```
 ///
+/// The following produces a compilation failure because `u32` has 4 times the
+/// size of `u8`:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// assert_eq_size!(u32, u8);
+/// # }
+/// ```
+///
 /// [`usize`]: https://doc.rust-lang.org/std/primitive.usize.html
 /// [`u64`]: https://doc.rust-lang.org/std/primitive.u64.html
 /// [`u32`]: https://doc.rust-lang.org/std/primitive.u32.html
@@ -123,12 +160,12 @@ macro_rules! assert_eq_size {
     };
 }
 
-/// Asserts at compile-time that the values pointed to have equal sizes.
+/// Asserts that values pointed to are equal in size.
 ///
 /// This especially is useful for when coercing pointers between different types
 /// and ensuring the underlying values are the same size.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```
 /// # #[macro_use]
@@ -138,25 +175,44 @@ macro_rules! assert_eq_size {
 /// }
 /// # fn main() {}
 /// ```
+///
+/// Byte arrays of different lengths have different sizes:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// static BYTES: &[u8; 4] = &[
+///     /* ... */
+///     # 0; 4
+/// ];
+///
+/// static TABLE: &[u8; 16] = &[
+///     /* ... */
+///     # 0; 16
+/// ];
+///
+/// assert_eq_size_ptr!(BYTES, TABLE);
+/// # }
+/// ```
 #[macro_export]
 macro_rules! assert_eq_size_ptr {
     ($x:expr, $($xs:expr),+ $(,)*) => {
-        #[allow(unknown_lints, forget_copy, unused_unsafe, useless_transmute)]
-        unsafe {
+        #[allow(unknown_lints, unsafe_code, forget_copy, useless_transmute)]
+        || unsafe {
             use $crate::_core::{mem, ptr};
             let mut copy = ptr::read($x);
             $(ptr::write(&mut copy, mem::transmute(ptr::read($xs)));)+
             mem::forget(copy);
-        }
+        };
     }
 }
 
-/// Asserts at compile-time that the values have equal sizes.
+/// Asserts that values are equal in size.
 ///
 /// This macro doesn't consume its arguments and thus works for
 /// non-[`Clone`]able values.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```
 /// # #[macro_use]
@@ -169,9 +225,15 @@ macro_rules! assert_eq_size_ptr {
 ///
 /// assert_eq_size_val!(x, y);
 /// assert_eq_size_val!(x, y, 0u8);
+/// # }
+/// ```
 ///
-/// // Fails to compile:
-/// // assert_eq_size_val!(x, 0u32);
+/// Even though both values are 0, they are of types with different sizes:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// assert_eq_size_val!(0u8, 0u32);
 /// # }
 /// ```
 ///
@@ -183,12 +245,12 @@ macro_rules! assert_eq_size_val {
     }
 }
 
-/// Asserts at compile-time that the constant expression evaluates to `true`.
+/// Asserts that constant expressions evaluate to `true`.
 ///
 /// There also exists [`const_assert_eq`](macro.const_assert_eq.html) for
 /// validating whether a sequence of expressions are equal to one another.
 ///
-/// # Example
+/// # Examples
 ///
 /// Constant expressions can be ensured to have certain properties via this
 /// macro If the expression evaluates to `false`, the file will fail to compile.
@@ -207,12 +269,18 @@ macro_rules! assert_eq_size_val {
 /// fn main() {
 ///     const_assert!(2 + 2 == 4);
 ///     const_assert!(FIVE - FIVE == 0);
-///
-///     // Produces a compilation failure:
-///     // const_assert!(1 >= 2);
 /// }
 /// ```
 ///
+/// Some expressions are blatantly false:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// const_assert!(1 >= 2);
+/// # }
+/// ```
+///
 /// [static_assert]: http://en.cppreference.com/w/cpp/language/static_assert
 #[macro_export]
 macro_rules! const_assert {
@@ -226,9 +294,9 @@ macro_rules! const_assert {
     };
 }
 
-/// Asserts at compile-time that the constants are equal in value.
+/// Asserts that constants are equal in value.
 ///
-/// # Example
+/// # Examples
 ///
 /// Works as a shorthand for `const_assert!(a == b)`:
 ///
@@ -238,14 +306,20 @@ macro_rules! const_assert {
 /// const TWO: usize = 2;
 /// const_assert_eq!(two; TWO * TWO, TWO + TWO, 4);
 ///
-/// // Fails to compile (same label):
-/// // const_assert_eq!(two; TWO, TWO);
-///
 /// fn main() {
 ///     const NUM: usize = 32;
 ///     const_assert_eq!(NUM + NUM, 64);
 /// }
 /// ```
+///
+/// Just because 2 × 2 = 2 + 2 doesn't mean it holds true for other numbers:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// const_assert_eq!(4 + 4, 4 * 4);
+/// # }
+/// ```
 #[macro_export]
 macro_rules! const_assert_eq {
     ($x:expr, $($xs:expr),+ $(,)*) => {
@@ -256,7 +330,7 @@ macro_rules! const_assert_eq {
     };
 }
 
-/// Asserts at compile-time that the traits are object-safe.
+/// Asserts that the traits are object-safe.
 ///
 /// This is useful for when changes are made to a trait that accidentally
 /// prevent it from being used as an object. Such a case would be adding a
@@ -264,32 +338,46 @@ macro_rules! const_assert_eq {
 /// unnoticed, that mistake will affect crate users and break both forward and
 /// backward compatibility.
 ///
-/// # Example
+/// # Examples
 ///
 /// ```
 /// # #[macro_use]
 /// # extern crate static_assertions;
 /// assert_obj_safe!(basic; Send, Sync, AsRef<str>);
 ///
-/// trait MySafeTrait {
-///     fn foo(&self) -> u32;
+/// mod inner {
+///     // Works with traits that are not in the calling module
+///     pub trait BasicTrait {
+///         fn foo(&self);
+///     }
 /// }
 ///
-/// trait MyUnsafeTrait {
-///     fn bar<T>(&self) -> T;
+/// trait MySafeTrait {
+///     fn bar(&self) -> u32;
 /// }
 ///
 /// fn main() {
 ///     assert_obj_safe!(MySafeTrait);
-///
-///     // Produces a compilation failure:
-///     // assert_obj_safe!(MyUnsafeTrait);
+///     assert_obj_safe!(inner::BasicTrait);
 /// }
 /// ```
+///
+/// Generics without `where Self: Sized` are not allowed in object-safe traits:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// trait MyUnsafeTrait {
+///     fn baz<T>(&self) -> T;
+/// }
+///
+/// # fn main() {
+/// assert_obj_safe!(MyUnsafeTrait);
+/// # }
+/// ```
 #[macro_export]
 macro_rules! assert_obj_safe {
     ($($xs:ty),+ $(,)*) => {
-        $(let _: Option<&$xs> = None;)+
+        $(let _: &$xs;)+
     };
     ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
@@ -297,13 +385,44 @@ macro_rules! assert_obj_safe {
     };
 }
 
-/// Asserts at compile-time that the type has the given fields.
+/// Asserts that the type has the given fields.
 ///
-/// This is useful for when types have odd fields as a result of `#[cfg]`.
+/// # Examples
+///
+/// This may be used when types have odd fields as a result of `#[cfg]`.
+///
+/// ```
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// struct Ty {
+///     #[cfg(windows)]
+///     value: u8,
+///     #[cfg(not(windows))]
+///     value: usize,
+/// }
+///
+/// /* ... */
+///
+/// // Always have `value` regardless of OS
+/// assert_fields!(Ty, value);
+/// # }
+/// ```
+///
+/// Range does not have a field named `middle`:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// use std::ops::Range;
+///
+/// assert_fields!(Range<u32>, middle);
+/// # }
+/// ```
 #[macro_export]
 macro_rules! assert_fields {
     ($t:path, $($f:ident),+) => {
-        $(let $t { $f: _, .. };)+
+        #[allow(unknown_lints, unneeded_field_pattern)]
+        { $(let $t { $f: _, .. };)+ }
     };
     ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
@@ -311,23 +430,26 @@ macro_rules! assert_fields {
     };
 }
 
-/// Asserts at compile-time that the type implements the given traits.
+/// Asserts that the type implements the given traits.
 ///
 /// # Examples
 ///
 /// Can be used to ensure types implement [`Send`], [`Sync`], and other traits:
 ///
 /// ```
-/// # #[macro_use]
-/// # extern crate static_assertions;
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {}
 /// assert_impl!(str; String, Send, Sync, From<&'static str>);
 /// assert_impl!(vec; &'static [u8], Into<Vec<u8>>);
+/// ```
 ///
-/// fn main() {
-///     // Produces a compilation failure:
-///     // `*const u8` cannot be sent between threads safely
-///     // assert_impl!(*const u8, Send);
-/// }
+/// Raw pointers cannot be sent between threads safely:
+///
+/// ```compile_fail
+/// # #[macro_use] extern crate static_assertions;
+/// # fn main() {
+/// assert_impl!(*const u8, Send);
+/// # }
 /// ```
 ///
 /// [`Send`]: https://doc.rust-lang.org/std/marker/trait.Send.html
@@ -335,10 +457,10 @@ macro_rules! assert_fields {
 #[macro_export]
 macro_rules! assert_impl {
     ($x:ty, $($t:path),+ $(,)*) => {
-        $({
-            fn assert_impl<T: ?Sized + $t>() {}
+        {
+            fn assert_impl<T>() where T: ?Sized $(+ $t)+ {}
             assert_impl::<$x>();
-        })+
+        }
     };
     ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
